# CycloMonitor Exceptions
# CycloMonitor Copyright (C) 2023 Nathaniel Greenwell
# This program comes with ABSOLUTELY NO WARRANTY; for details see main.py
class AutoUpdateError(Exception):
    pass


class LogRequested(Exception):
    pass
